@@ -1,4 +1,4 @@
-/*	$OpenBSD: cert.c,v 1.28 2021/03/05 17:15:19 claudio Exp $ */
+/*	$OpenBSD: cert.c,v 1.31 2021/07/13 18:39:39 job Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -46,27 +46,6 @@ struct	parse {
 	const char	*fn; /* currently-parsed file */
 };
 
-/*
- * Wrapper around ASN1_get_object() that preserves the current start
- * state and returns a more meaningful value.
- * Return zero on failure, non-zero on success.
- */
-static int
-ASN1_frame(struct parse *p, size_t sz,
-	const unsigned char **cnt, long *cntsz, int *tag)
-{
-	int	 ret, pcls;
-
-	assert(cnt != NULL && *cnt != NULL);
-	assert(sz > 0);
-	ret = ASN1_get_object(cnt, cntsz, tag, &pcls, sz);
-	if ((ret & 0x80)) {
-		cryptowarnx("%s: ASN1_get_object", p->fn);
-		return 0;
-	}
-	return ASN1_object_size((ret & 0x01) ? 2 : 0, *cntsz, *tag);
-}
-
 /*
  * Append an IP address structure to our list of results.
  * This will also constrain us to having at most one inheritence
@@ -268,7 +247,7 @@ sbgp_sia_resource_entry(struct parse *p,
 
 	d = t->value.asn1_string->data;
 	dsz = t->value.asn1_string->length;
-	if (!ASN1_frame(p, dsz, &d, &plen, &ptag))
+	if (!ASN1_frame(p->fn, dsz, &d, &plen, &ptag))
 		goto out;
 
 	/*
@@ -436,7 +415,7 @@ sbgp_asrange(struct parse *p, const unsigned char *d, size_t dsz)
 		goto out;
 	}
 	if (!as_id_parse(t->value.integer, &as.range.min)) {
-		warnx("%s: RFC 3770 section 3.2.3.8 (via RFC 1930): "
+		warnx("%s: RFC 3779 section 3.2.3.8 (via RFC 1930): "
 		    "malformed AS identifier", p->fn);
 		return 0;
 	}
@@ -449,7 +428,7 @@ sbgp_asrange(struct parse *p, const unsigned char *d, size_t dsz)
 		goto out;
 	}
 	if (!as_id_parse(t->value.integer, &as.range.max)) {
-		warnx("%s: RFC 3770 section 3.2.3.8 (via RFC 1930): "
+		warnx("%s: RFC 3779 section 3.2.3.8 (via RFC 1930): "
 		    "malformed AS identifier", p->fn);
 		return 0;
 	}
@@ -484,12 +463,12 @@ sbgp_asid(struct parse *p, const ASN1_INTEGER *i)
 	as.type = CERT_AS_ID;
 
 	if (!as_id_parse(i, &as.id)) {
-		warnx("%s: RFC 3770 section 3.2.3.10 (via RFC 1930): "
+		warnx("%s: RFC 3779 section 3.2.3.10 (via RFC 1930): "
 		    "malformed AS identifier", p->fn);
 		return 0;
 	}
 	if (as.id == 0) {
-		warnx("%s: RFC 3770 section 3.2.3.10 (via RFC 1930): "
+		warnx("%s: RFC 3779 section 3.2.3.10 (via RFC 1930): "
 		    "AS identifier zero is reserved", p->fn);
 		return 0;
 	}
@@ -564,7 +543,7 @@ sbgp_asnum(struct parse *p, const unsigned char *d, size_t dsz)
 				goto out;
 			break;
 		default:
-			warnx("%s: RFC 3779 section 3.2.3.4: IPAddressOrRange: "
+			warnx("%s: RFC 3779 section 3.2.3.5: ASIdOrRange: "
 			    "want ASN.1 sequence or integer, have %s (NID %d)",
 			    p->fn, ASN1_tag2str(tt->type), tt->type);
 			goto out;
@@ -666,7 +645,7 @@ sbgp_assysnum(struct parse *p, X509_EXTENSION *ext)
 
 		d = t->value.asn1_string->data;
 		dsz = t->value.asn1_string->length;
-		if (!ASN1_frame(p, dsz, &d, &plen, &ptag))
+		if (!ASN1_frame(p->fn, dsz, &d, &plen, &ptag))
 			goto out;
 
 		/* Ignore bad AS identifiers and RDI entries. */
