@@ -1,4 +1,4 @@
-/*	$OpenBSD: cms.c,v 1.7 2020/04/02 09:16:43 claudio Exp $ */
+/*	$OpenBSD: cms.c,v 1.9 2021/07/13 18:39:39 job Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -36,17 +36,16 @@
  */
 unsigned char *
 cms_parse_validate(X509 **xp, const char *fn,
-    const char *oid, const unsigned char *dgst, size_t *rsz)
+    const char *oid, size_t *rsz)
 {
 	const ASN1_OBJECT	*obj;
 	ASN1_OCTET_STRING	**os = NULL;
-	BIO			*bio = NULL, *shamd;
+	BIO			*bio = NULL;
 	CMS_ContentInfo		*cms;
 	FILE			*f;
-	char			 buf[128], mdbuf[EVP_MAX_MD_SIZE];
+	char			 buf[128];
 	int			 rc = 0, sz;
 	STACK_OF(X509)		*certs = NULL;
-	EVP_MD			*md;
 	unsigned char		*res = NULL;
 
 	*rsz = 0;
@@ -66,48 +65,11 @@ cms_parse_validate(X509 **xp, const char *fn,
 		return NULL;
 	}
 
-	/*
-	 * If we have a digest specified, create an MD chain that will
-	 * automatically compute a digest during the CMS creation.
-	 */
-
-	if (dgst != NULL) {
-		if ((shamd = BIO_new(BIO_f_md())) == NULL)
-			cryptoerrx("BIO_new");
-		if (!BIO_set_md(shamd, EVP_sha256()))
-			cryptoerrx("BIO_set_md");
-		if ((bio = BIO_push(shamd, bio)) == NULL)
-			cryptoerrx("BIO_push");
-	}
-
 	if ((cms = d2i_CMS_bio(bio, NULL)) == NULL) {
 		cryptowarnx("%s: RFC 6488: failed CMS parse", fn);
 		goto out;
 	}
 
-	/*
-	 * If we have a digest, find it in the chain (we'll already have
-	 * made it, so assert otherwise) and verify it.
-	 */
-
-	if (dgst != NULL) {
-		shamd = BIO_find_type(bio, BIO_TYPE_MD);
-		assert(shamd != NULL);
-
-		if (!BIO_get_md(shamd, &md))
-			cryptoerrx("BIO_get_md");
-		assert(EVP_MD_type(md) == NID_sha256);
-
-		if ((sz = BIO_gets(shamd, mdbuf, EVP_MAX_MD_SIZE)) < 0)
-			cryptoerrx("BIO_gets");
-		assert(sz == SHA256_DIGEST_LENGTH);
-
-		if (memcmp(mdbuf, dgst, SHA256_DIGEST_LENGTH)) {
-			warnx("%s: RFC 6488: bad message digest", fn);
-			goto out;
-		}
-	}
-
 	/*
 	 * The CMS is self-signed with a signing certifiate.
 	 * Verify that the self-signage is correct.
@@ -182,3 +144,61 @@ out:
 
 	return res;
 }
+
+/*
+ * Wrapper around ASN1_get_object() that preserves the current start
+ * state and returns a more meaningful value.
+ * Return zero on failure, non-zero on success.
+ */
+int
+ASN1_frame(const char *fn, size_t sz,
+	const unsigned char **cnt, long *cntsz, int *tag)
+{
+	int	 ret, pcls;
+
+	ret = ASN1_get_object(cnt, cntsz, tag, &pcls, sz);
+	if ((ret & 0x80)) {
+		cryptowarnx("%s: ASN1_get_object", fn);
+		return 0;
+	}
+	return ASN1_object_size((ret & 0x01) ? 2 : 0, *cntsz, *tag);
+}
+
+/*
+ * Check the version field in eContent.
+ * Returns -1 on failure, zero on success.
+ */
+int
+cms_econtent_version(const char *fn, const unsigned char **d, size_t dsz,
+	long *version)
+{
+	ASN1_INTEGER	*aint = NULL;
+	long		 plen;
+	int		 ptag, rc = -1;
+
+	if (!ASN1_frame(fn, dsz, d, &plen, &ptag))
+		goto out;
+	if (ptag != 0) {
+		warnx("%s: eContent version: expected explicit tag [0]", fn);
+		goto out;
+	}
+
+	aint = d2i_ASN1_INTEGER(NULL, d, plen);
+	if (aint == NULL) {
+		cryptowarnx("%s: eContent version: failed d2i_ASN1_INTEGER",
+		    fn);
+		goto out;
+	}
+
+	*version = ASN1_INTEGER_get(aint);
+	if (*version < 0) {
+		warnx("%s: eContent version: expected positive integer, got:"
+		    " %ld", fn, *version);
+		goto out;
+	}
+
+	rc = 0;
+out:
+	ASN1_INTEGER_free(aint);
+	return rc;
+}
