@@ -1,4 +1,4 @@
-/*	$OpenBSD: io.c,v 1.9 2020/09/12 15:46:48 claudio Exp $ */
+/*	$OpenBSD: io.c,v 1.13 2021/03/04 13:01:41 claudio Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -16,14 +16,17 @@
  */
 
 #include <sys/queue.h>
+#include <sys/socket.h>
 
 #include <assert.h>
 #include <err.h>
+#include <errno.h>
 #include <fcntl.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <imsg.h>
 
 #include "extern.h"
 
@@ -49,85 +52,38 @@ io_socket_nonblocking(int fd)
 		err(1, "fcntl");
 }
 
-/*
- * Blocking write of a binary buffer.
- * Buffers of length zero are simply ignored.
- */
-void
-io_simple_write(int fd, const void *res, size_t sz)
-{
-	ssize_t	 ssz;
-
-	if (sz == 0)
-		return;
-	if ((ssz = write(fd, res, sz)) == -1)
-		err(1, "write");
-	else if ((size_t)ssz != sz)
-		errx(1, "write: short write");
-}
-
 /*
  * Like io_simple_write() but into a buffer.
  */
 void
-io_simple_buffer(char **b, size_t *bsz,
-	size_t *bmax, const void *res, size_t sz)
+io_simple_buffer(struct ibuf *b, const void *res, size_t sz)
 {
-
-	if (*bsz + sz > *bmax) {
-		if ((*b = realloc(*b, *bsz + sz)) == NULL)
-			err(1, NULL);
-		*bmax = *bsz + sz;
-	}
-
-	memcpy(*b + *bsz, res, sz);
-	*bsz += sz;
+	if (ibuf_add(b, res, sz) == -1)
+		err(1, NULL);
 }
 
 /*
- * Like io_buf_write() but into a buffer.
+ * Add a sz sized buffer into the io buffer.
  */
 void
-io_buf_buffer(char **b, size_t *bsz,
-	size_t *bmax, const void *p, size_t sz)
+io_buf_buffer(struct ibuf *b, const void *p, size_t sz)
 {
-
-	io_simple_buffer(b, bsz, bmax, &sz, sizeof(size_t));
+	if (ibuf_add(b, &sz, sizeof(size_t)) == -1)
+		err(1, NULL);
 	if (sz > 0)
-		io_simple_buffer(b, bsz, bmax, p, sz);
+		if (ibuf_add(b, p, sz) == -1)
+			err(1, NULL);
 }
 
 /*
- * Write a binary buffer of the given size, which may be zero.
+ * Add a string into the io buffer.
  */
 void
-io_buf_write(int fd, const void *p, size_t sz)
+io_str_buffer(struct ibuf *b, const char *p)
 {
+	size_t sz = (p == NULL) ? 0 : strlen(p);
 
-	io_simple_write(fd, &sz, sizeof(size_t));
-	io_simple_write(fd, p, sz);
-}
-
-/*
- * Like io_str_write() but into a buffer.
- */
-void
-io_str_buffer(char **b, size_t *bsz, size_t *bmax, const char *p)
-{
-	size_t	 sz = (p == NULL) ? 0 : strlen(p);
-
-	io_buf_buffer(b, bsz, bmax, p, sz);
-}
-
-/*
- * Write a NUL-terminated string, which may be zero-length.
- */
-void
-io_str_write(int fd, const char *p)
-{
-	size_t	 sz = (p == NULL) ? 0 : strlen(p);
-
-	io_buf_write(fd, p, sz);
+	io_buf_buffer(b, p, sz);
 }
 
 /*
@@ -175,7 +131,7 @@ io_buf_read_alloc(int fd, void **res, size_t *sz)
 }
 
 /*
- * Read a string (which may just be \0 and zero-length), allocating
+ * Read a string (returns NULL for zero-length strings), allocating
  * space for it.
  */
 void
@@ -184,7 +140,86 @@ io_str_read(int fd, char **res)
 	size_t	 sz;
 
 	io_simple_read(fd, &sz, sizeof(size_t));
+	if (sz == 0) {
+		*res = NULL;
+		return;
+	}
 	if ((*res = calloc(sz + 1, 1)) == NULL)
 		err(1, NULL);
 	io_simple_read(fd, *res, sz);
 }
+
+/*
+ * Read data from socket but receive a file descriptor at the same time.
+ */
+int
+io_recvfd(int fd, void *res, size_t sz)
+{
+	struct iovec iov;
+	struct msghdr msg;
+	struct cmsghdr *cmsg;
+	union {
+		struct cmsghdr	hdr;
+		char		buf[CMSG_SPACE(sizeof(int))];
+	} cmsgbuf;
+	int outfd = -1;
+	char *b = res;
+	ssize_t n;
+
+	memset(&msg, 0, sizeof(msg));
+	memset(&cmsgbuf, 0, sizeof(cmsgbuf));
+
+	iov.iov_base = res;
+	iov.iov_len = sz;
+
+	msg.msg_iov = &iov;
+	msg.msg_iovlen = 1;
+	msg.msg_control = &cmsgbuf.buf;
+	msg.msg_controllen = sizeof(cmsgbuf.buf);
+
+	while ((n = recvmsg(fd, &msg, 0)) == -1) {
+		if (errno == EINTR)
+			continue;
+		err(1, "recvmsg");
+	}
+
+	if (n == 0)
+		errx(1, "recvmsg: unexpected end of file");
+
+	for (cmsg = CMSG_FIRSTHDR(&msg); cmsg != NULL;
+	    cmsg = CMSG_NXTHDR(&msg, cmsg)) {
+		if (cmsg->cmsg_level == SOL_SOCKET &&
+		    cmsg->cmsg_type == SCM_RIGHTS) {
+			int i, j, f;
+
+			j = ((char *)cmsg + cmsg->cmsg_len -
+			    (char *)CMSG_DATA(cmsg)) / sizeof(int);
+			for (i = 0; i < j; i++) {
+				f = ((int *)CMSG_DATA(cmsg))[i];
+				if (i == 0)
+					outfd = f;
+				else
+					close(f);
+			}
+		}
+	}
+
+	b += n;
+	sz -= n;
+	while (sz > 0) {
+		/* short receive */
+		n = recv(fd, b, sz, 0);
+		if (n == -1) {
+			if (errno == EINTR)
+				continue;
+			err(1, "recv");
+		}
+		if (n == 0)
+			errx(1, "recv: unexpected end of file");
+
+		b += n;
+		sz -= n;
+	}
+
+	return outfd;
+}
